/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
 * req_getcred.c
 *
 * This file contains function relating to the PBS credential system,
 * it includes the major functions:
 *   req_authenuser - Authenticate a user connection based on pbs_iff  (new)
 *   req_connect    - validate the credential in a Connection Request (old)
 */
#include <pbs_config.h>   /* the master config generated by configure */

#include <sys/types.h>
#include "libpbs.h"
#include "server_limits.h"
#include "list_link.h"
#include "attribute.h"
#include "credential.h"
#include "net_connect.h"
#include "batch_request.h"

static char ident[] = "@(#) $RCSfile: req_getcred.c,v $ $Revision: 2.0 $";

/* External Global Data Items Referenced */

extern time_t time_now;
extern struct connection svr_conn[];

/* Global Data Home in this file */

struct credential conn_credent[PBS_NET_MAX_CONNECTIONS];
	/* there is one per possible connectinn */

/*
 * req_connect - process a Connection Request
 * 	Almost does nothing.
 */

void req_connect(preq)
	struct batch_request *preq;
{
	int  sock = preq->rq_conn;

	if (svr_conn[sock].cn_authen == 0) {
		reply_ack(preq);
	} else
		req_reject(PBSE_BADCRED, 0, preq);
}

/*
 * req_authenuser - Authenticate a user connection based on the (new)
 *	pbs_iff information.  Pbs_iff will contact the server on a privileged
 *	port and identify the user who has made an existing, but yet unused,
 *	non-privileged connection.  This connection is marked as authenticated.
 */

void req_authenuser(preq)
	struct batch_request *preq;
{
	int s;

	/*
	 * find the socket whose client side is bound to the port named 
	 * in the request
	 */

	for (s=0; s<PBS_NET_MAX_CONNECTIONS; ++s) {
	    if (preq->rq_ind.rq_authen.rq_port == svr_conn[s].cn_port) {
		if (svr_conn[s].cn_authen == 0) {
			
		    (void)strcpy(conn_credent[s].username, preq->rq_user);
		    (void)strcpy(conn_credent[s].hostname, preq->rq_host);
		    /* time stamp just for the record */
		    conn_credent[s].timestamp = time_now;
		    svr_conn[s].cn_authen = PBS_NET_CONN_AUTHENTICATED;
		}
		reply_ack(preq);
		return;
	    }
	}
	req_reject(PBSE_BADCRED, 0, preq);
}
